Ext.namespace('Zarafa.plugins.smime');

/**
 * @class Zarafa.plugins.smime.SmimePlugin
 * @extends Zarafa.core.Plugin
 */
Zarafa.plugins.smime.SmimePlugin = Ext.extend(Zarafa.core.Plugin, {
	/*
	 * Called after constructor.
	 * Registers insertion points.
	 * @protected
	 */
	initPlugin : function()
	{	
		Zarafa.plugins.smime.SmimePlugin.superclass.initPlugin.apply(this, arguments);

		// S/MIME button in mailcreatecontentpanel
		this.registerInsertionPoint('context.mail.mailcreatecontentpanel.toolbar.options', this.showSignButton, this);
		this.registerInsertionPoint('context.mail.mailcreatecontentpanel.toolbar.options', this.showEncryptButton, this);

		// S/MIME Settings widget insertion point
		this.registerInsertionPoint('context.settings.categories', this.createSettingsCategories, this);

		// Insertion point which shows the verified/decrypted status
		this.registerInsertionPoint('previewpanel.toolbar.detaillinks', this.showSmimeInfo, this);

		// S/MIME Icons
		this.registerInsertionPoint('context.mail.gridrow', this.showMessageClass , this);
		this.registerInsertionPoint('context.mail.griddefaultcolumn', this.showDefaultColumn, this);
			
		Zarafa.core.data.SharedComponentType.addProperty('plugin.smime.dialog.passphrasewindow');
	},

	/**
	 * Create a category in settings for S/MIME
	 *
	 * @return {smimesettingscategory} 
	 */
	createSettingsCategories : function(insertionName, settingsMainPanel, settingsContext) 
	{
		return [{
			xtype : 'smime.settingssmimecategory',
			settingsContext: settingsContext
		}];
	},

	/**
	 * Displays S/MIME information in the previewpanel  
	 *
	 * @return {Object} a box which on record update displays S/MIME information
	 */
	showSmimeInfo : function() 
	{
		return {
			xtype: 'button',
			plugins : [ 'zarafa.recordcomponentupdaterplugin' ],
			autoEl: {
				tag: 'div',
				ref: 'smimeInfoBox'
			},
			scope : this,
			update : this.onSmimeInfo,
			handler : this.onSmimeButton
		};
	},

	/**
	 * Create button which sets the MessageClass to IPM.Note.SMIME.MultiPartSigned
	 * when creating an email.
	 * If certificate is unlocked we can display the Sign button, else a blank label
	 *
	 * @return {Config} creates a button for signing email
	 */
	showSignButton : function() 
	{
		return {
			xtype : 'button',
			text : _('Sign'),
			tooltip: {
				title: _('Sign message'),
				text: _('Ensure the authenticity of the message by adding a digital signature to this message. Any changes to the message will invalidate the signature.')
			},
			iconCls : 'icon_smime_sign',
			handler : this.onSignButton,
			scope : this
		};
	},

	/**
	 * Create button which sets the MessageClass to IPM.Note.SMIME
	 * when creating an email in context.mail.mailcreatecontentpanel.toolbar.option
	 *
	 * @return {Object} creates a button for encrypting an email
	 */
	showEncryptButton : function() 
	{
		return {
			xtype : 'button',
			text : _('Encrypt'),
			tooltip: {
				title: _('Encrypt message'),
				text: _('Ensure the privacy of the message by encrypting its contents. Only the recipient of the message will be able to open it.')
			},
			iconCls : 'icon_smime_encrypt',
			handler : this.onEncryptButton,
			scope : this
		};
	},

	/**
	 * Handler for the button which is displayed when a encrypted / signed message is openend
	 * When an encrypted message is opened, we will send a request to unlock the certificate.
	 * When an signed email is openend, we will show a popup with extra information about the signed message
	 */
	onSmimeButton: function(button, config)
	{
		var smimeInfo = button.record.get('smime');
		if(button.record.isOpened() && smimeInfo) {
			switch(smimeInfo.type) {
				case 'encrypted':
					if (smimeInfo.success !== Zarafa.plugins.smime.SMIME_STATUS_GOOD) {
						var user = container.getUser();
						container.getRequest().singleRequest(
							'pluginsmimemodule',
							'certificate',
							{ 
							  'user' : user.getSMTPAddress(),
							  'sessionid' : user.getSessionId()
							},
							new Zarafa.plugins.smime.data.SmimeResponseHandler({
								successCallback : this.onCertificateCallback.createDelegate(button.record)
							})
						);
					}
					break;
				case 'signed':
				case 'encryptsigned':
					container.getNotifier().notify(Zarafa.plugins.smime.SmimeText.getPopupStatus(smimeInfo.success), 
							_('What\'s going on with my email?'), Zarafa.plugins.smime.SmimeText.getPopupText(smimeInfo.info));
					break;
			}
		}
	},

	/**
	 * Function which displays information in the previewpanel.
	 * In the case of a encrypted message, we either show a button to unlock the certificate
	 * or shows the message that the message has been decrypted.
	 * If the message is signed we display information depending on the state of the verification.
	 * 
	 * @param {Zarafa.core.data.IPMRecord} record record which is displayed
	 * @param {Boolean} contentReset force the component to perform a full update of the data.
	 */
	onSmimeInfo : function(record, resetContent) 
	{
		// Set button.record for use in onSmimeButton
		this.record = record;
		var smimeInfoBox = this.getEl();
		
		// Set smimeBox to empty value by default, to override previous S/MIME message text
		smimeInfoBox.update("");
		smimeInfoBox.removeClass('smime-info-good');
		smimeInfoBox.removeClass('smime-info-fatal');
		smimeInfoBox.removeClass('smime-info-partial');

		// retrieve smime json object
		var smimeInfo = record.get('smime');
		if(record.isOpened() && smimeInfo) {
			var sender = record.getSender();

			// FIXME: refactor success, to status since that's probably a better description of the variable
			smimeInfoBox.addClass(Zarafa.plugins.smime.SmimeText.getStatusMessageClass(smimeInfo.success));
			var message = Zarafa.plugins.smime.SmimeText.getMessageInfo(smimeInfo.info);
			switch(smimeInfo.type) {
				case 'encrypted':
					// Empty smimeInfoBox
					if( smimeInfo.success === Zarafa.plugins.smime.SMIME_STATUS_BAD) {
						smimeInfoBox.update('<div class="icon_smime_encr_content"></div> ' + message);
					} else {
						smimeInfoBox.update(String.format(' {0} &lt{1}&gt <div class="icon_smime_decr_content"></div> {2}', sender.get('display_name'), sender.get('smtp_address'), message));
						// Force the Attachmentlinks component to update, to view the attachments
						this.ownerCt.findByType('zarafa.attachmentlinks')[0].update(record, true);
					}
					break;
				case 'signed':
					smimeInfoBox.update(String.format('{0} &lt{1}&gt <div class="icon_smime_sign_content"></div> {2}', sender.get('display_name'), sender.get('smtp_address'), message));
					break;
				case 'encryptsigned':
					smimeInfoBox.update(String.format('{0} &lt{1}&gt <div class="icon_smime_sign_content"></div> <div class="icon_smime_decr_content"></div> {2}', sender.get('display_name'), sender.get('smtp_address'), message));
					if(smimeInfo.success !== Zarafa.plugins.smime.SMIME_STATUS_BAD) {
						// Force the Attachmentlinks component to update, to view the attachments
						this.ownerCt.findByType('zarafa.attachmentlinks')[0].update(record, true);
					}
					break;
			}
		}
	},

	/**
	 * Function which collects all the recipients smtp_addresses in a list and sets them as 'smime'
	 * property in the mailrecord. Because the hook in PHP doesn't have updated recipienttable yet.
	 *
	 * @param {Zarafa.mailcreatecontentpanel} dialog 
	 * @param {Zarfa.core.data.IPMRecord} record The record which is going to be send
	 * @return {Boolean} returns false if public key isn't find and stops the record from being send
	 *
	 */
	onBeforeSendRecord : function(dialog, record) {
		// Get list of recipients and set them in the 'smime' property
		var recipientStore = record.getRecipientStore();
		var recipients = [];
		recipientStore.each(function(recip) {
			recipients.push(recip.get('smtp_address'));
		}, this);
		dialog.record.set('smime', recipients);

		return true;
	},

	/**
	 * Handler for the encrypt button, when clicked it sets the message_class to encrypted.
	 * If we press the button again we reset the message_class. When a user already has signed an email
	 * we set a special message_class on it to sign+encrypt
	 *
	 * @param {Ext.button} button
	 * @param {Object} config
	 */
	onEncryptButton : function(button, config) 
	{
		var owner = button.ownerCt;
		var record = owner.record;
		if(record) {
			switch(record.get('message_class')) {
				// Sign and Encrypt
				case 'IPM.Note.SMIME.MultipartSigned':
					record.set('message_class', 'IPM.Note.SMIME.SignedEncrypt');
					button.setIconClass('icon_smime_encrypt_selected');

					// Add event to check if all recipients have a public key
					owner.dialog.on('beforesendrecord', this.onBeforeSendRecord ,this);
					break;
				// We want to encrypt
				case 'IPM.Note': 
					button.setIconClass('icon_smime_encrypt_selected');
					record.set('message_class', 'IPM.Note.SMIME');

					// Add event to check if all recipients have a public key
					owner.dialog.on('beforesendrecord', this.onBeforeSendRecord ,this);
					break;
				// Unselecting encrypt functionality
				case 'IPM.Note.SMIME': 
				case 'IPM.Note.SMIME.SignedEncrypt':
					if (record.get('message_class') === 'IPM.Note.SMIME.SignedEncrypt') {
						record.set('message_class', 'IPM.Note.SMIME.MultipartSigned');
					} else {
						record.set('message_class', 'IPM.Note');
					}

					button.setIconClass('icon_smime_encrypt');
					// Reset smime property
					record.set('smime','')
					// Reset send action, otherwise the saveRecord will trigger a send when the user deselects encryption
					record.actions = {}

					// Remove event
					owner.dialog.un('beforesendrecord', this.onBeforeSendRecord ,this);
					break;
			}
			owner.dialog.saveRecord();
		}
	},

	/**
	 * Handler for the sign button, when clicked it checks if the private certificate exists.
	 * If we have signing already set and click it again, we unset it.
	 * If we already set have encryption set, we set a special message_class for both sign+ecnrypt.
	 *
	 *
	 * @param {Ext.button} button
	 * @param {Object} config
	 */
	onSignButton : function(button, config) 
	{
		var owner = button.ownerCt;
		var record = owner.record;
		if(record) {
			var plugin = this;
			var user = container.getUser();
			var request = function() {
				container.getRequest().singleRequest(
					'pluginsmimemodule',
					'certificate',
					{ 
					  'user' : user.getSMTPAddress(),
					  'sessionid' : user.getSessionId()
					},
					new Zarafa.plugins.smime.data.SmimeResponseHandler({
						successCallback : plugin.onCertificateCallback.createDelegate(button)
					})
				);
			}
			switch(record.get('message_class')) {
				// We want to sign
				case 'IPM.Note':
					button.message_class = 'IPM.Note.SMIME.MultipartSigned';
					request();
					break;
				// Encrypt + Sign
				case 'IPM.Note.SMIME':
					button.message_class = 'IPM.Note.SMIME.SignedEncrypt';
					request();
					break;
				case 'IPM.Note.SMIME.MultipartSigned':
					button.setIconClass('icon_smime_sign');
					record.set('message_class', 'IPM.Note');
					break;
				case 'IPM.Note.SMIME.SignedEncrypt':
					button.setIconClass('icon_smime_sign');
					record.set('message_class', 'IPM.Note.SMIME');
					break;
			}
		}
		owner.dialog.saveRecord();
	},

	/**
	 * successCallback function for the request to verify if a private certificate exists in the mapi userstore.
	 * If the response status is true the certificate exists and we will query the user for his passphrase.
	 *
	 * @param {Object} response Json object containing the response from PHP
	 */
	onCertificateCallback : function(response) {
		// TODO: improve functionality with less callbacks
		var btn = this;
		if(response.status) { 
		    	Zarafa.core.data.UIFactory.openLayerComponent(Zarafa.core.data.SharedComponentType['plugin.smime.dialog.passphrasewindow'], btn, {manager: Ext.WindowMgr});
		} else {
			container.getNotifier().notify('info.saved', _('S/MIME Message'), response.message);
		}
	},

	/*
	 * Bid for the type of shared component
	 * and the given record.
	 * This will bid on calendar.dialogs.importevents
	 * @param {Zarafa.core.data.SharedComponentType} type Type of component a context can bid for.
	 * @param {Zarafa.mail.dialogs.MailCreateContentPanel} owner Optionally passed panel
	 * @return {Number} The bid for the shared component
	 */
	bidSharedComponent : function(type, record) {
		var bid = -1;

		switch(type) {
			case Zarafa.core.data.SharedComponentType['plugin.smime.dialog.passphrasewindow']:
				bid = 1;
				break;
		}
		return bid;
	},

	/**
	 * Will return the reference to the shared component.
	 * Based on the type of component requested a component is returned.
	 * @param {Zarafa.core.data.SharedComponentType} type Type of component a context can bid for.
	 * @param {Zarafa.mail.dialogs.MailCreateContentPanel} owner Optionally passed panel
	 * @return {Ext.Component} Component
	 */
	getSharedComponent : function(type, record) {
		var component;

		switch(type) {
			case Zarafa.core.data.SharedComponentType['plugin.smime.dialog.passphrasewindow']:
				component = Zarafa.plugins.smime.dialogs.PassphraseWindow;
				break;
		}

		return component;
	},

	/**
	 * Shows the message class icon of signed or encrypted email in the defaultcolumn
	 *
	 * @param {string} insertionPoint name of insertion point
	 * @param {Zarafa.core.data.IPMRecord} record The record of a row
	 *
	 */
	showDefaultColumn : function(insertionPoint, record) 
	{
		return  {
			header : _('SMIME Message'),
			dataIndex : 'message_class',
			width : 12,
			sortable : false,
			renderer :  function(value, p, record) { 
				var messageClass = record.get('message_class'); 
				if(messageClass == 'IPM.Note.SMIME') {
					p.css = 'icon_smime_encrypt'; 
				} else if(messageClass == 'IPM.Note.SMIME.MultipartSigned') {
					p.css = 'icon_smime_sign'; 
				}
				return ''; 
			},
			fixed : true,
			tooltip : _('S/MIME Message')
		};
	},

	/**
	 * Shows the message class icon of signed or encrypted email in the non defaultcolumn
	 *
	 * @param {string} insertionPoint name of insertion point
	 * @param {Zarafa.core.data.IPMRecord} record The record of a row
	 * @return {string} column entry
	 *
	 */
	showMessageClass : function(insertionPoint, record) { 
		var messageClass = record.get('message_class');
		var icon = "";
		if(messageClass == 'IPM.Note.SMIME') {
			icon = 'icon_smime_encrypt';
		} 
		else if(messageClass == 'IPM.Note.SMIME.MultipartSigned') {
			icon = 'icon_smime_sign';
		}
		return String.format('<td style="width: 24px"><div class="grid_compact {0}" style="height: 24px; width: 24px;">{1}</div></td>', icon, "");
	}
});

// Add property to record to MailRecord, where extra information is stored about S/MIME messages
Zarafa.core.data.RecordFactory.addFieldToMessageClass('IPM.Note', [{name: 'smime', defaultValue: ''}]);

Zarafa.onReady(function() {
	Zarafa.plugins.smime.SMIME_STATUS_GOOD = 0;
	Zarafa.plugins.smime.SMIME_STATUS_PARTIAL = 2;
	Zarafa.plugins.smime.SMIME_STATUS_FATAL = 2;

	container.registerPlugin(new Zarafa.core.PluginMetaData({
		name : 'smime',
		displayName : _('S/MIME Plugin'),
		pluginConstructor : Zarafa.plugins.smime.SmimePlugin
	}));
});
Ext.namespace('Zarafa.plugins.smime');

/**
 * @class Zarafa.plugins.smime.SmimeText
 * Singleton which holds two functions with translation messages for certain status codes from PHP for S/MIME messages
 * One helper function to create a message.
 * @singleton
 */
Zarafa.plugins.smime.SmimeText = function() {
	return {
		/*
		 * Helper function to create the popup body message
		 *
		 * @param {String} text
		 * @return {String} popup text
		 */
		createMessage : function(text) {
			 return _("WebApp can verify digital signatures of emails. A successful verification reassures you that the message has not been tampered with and validates the identity of the sender.")
				+ "<br><br>" + _("You are seeing this message because the verification of the digital signature") + "<b> " + _(" has failed") + "</b>" + _(" for this message.") + "<br>"
				+ "<br><b>" + _("What caused this issue?") + "</b><br><br>"
				+ text
				+ "<br><br><b>" + _("What should I do?") + "</b><br><br>"
				+ _("You can continue to work. However, you should keep in mind that the identity and the authenticity of the message content could not be verified.")
				+ "<br><br>" + _("You can contact your helpdesk or system administrator if you are not sure what to do next.");
		},

		/**
		 * S/MIME Popup text
		 * Function which returns the text for the popup which is drawn when a users clicks on the smimeInfo button
		 *
		 * @param {Number} index status code from PHP
		 * @return {String} popup text
		 */
		getPopupText : function(index) 
		{
			switch(index) {
				// Verified succesfully
				case 0:
					return _("WebApp can verify digital signatures of emails. A successful verification reassures you ")
					  + _("that the message has not been tampered with and validates the identity of the sender.") + "<br><br>"
					  + _("The verification of the digital signature was successful for this email message.");
				// Could not verify, missing public certificate
				case 1:
					return Zarafa.plugins.smime.SmimeText.createMessage(_("WebApp could not find a public certificate for the recipient."));
			 	// Signature verified, but certificate expired
				case 2:
					return Zarafa.plugins.smime.SmimeText.createMessage(_("The identity of the sender and authenticity of the message content have been verified, but the certificate used to sign the message expired on [d-m-Y]."));
			 	// Signature could not be verified
				case 3:
					return Zarafa.plugins.smime.SmimeText.createMessage(_("The digital signature could not be verified for unknown reasons."));
				// Signature has been revoked
				case 4:
					return Zarafa.plugins.smime.SmimeText.createMessage(_("The digital certificate used to sign this message has been revoked (i.e. the sender has marked it as compromised)"));
				// The verification step with the Certificate Authority failed
				case 5:
					return Zarafa.plugins.smime.SmimeText.createMessage(_("The verification service of the certificate authority that signed the sender’s certificate is not available. The validity of the certificate could not be verified."));
				// Certificate does not support OCSP
				case 9: 
					return Zarafa.plugins.smime.SmimeText.createMessage(_("The revocation status of the digital certificate used to sign this email is unknown (Server is unavaliable or certificate does not support OCSP). The validity of the certificate could not be verified."));
				// OCSP check disabled
				case 10:
					return Zarafa.plugins.smime.SmimeText.createMessage(_("The revocation status of the digital certificate used to sign this email is disabled (OCSP). The validity of the certificate could not be verified."));
				// OCSP server offline
				case 11:
					return Zarafa.plugins.smime.SmimeText.createMessage(_("The certificate verification server (OCSP) is temporarly offline."));

			}
		},

		/**
		 * Function to retreive the status messages for the corresponding status code from PHP
		 * The translated text is drawn in the S/MIME previewpanel button
		 *
		 * @param {Number} index
		 * @return {String} text 
		 */
		getMessageInfo : function(index)
		{
			switch(index) {
				case 0:
					return _('Signature verified succesfully');
				case 1:
					return _('Could not verify signature, missing public certificate');
				case 2:
					return _('Signature verification succesfull, but certificate is expired');
				case 3:
					return _('Signature could not be verified');
				case 4:
					return _('Certificate has been revoked');
				case 5:
					return _('Verification with Certificate Authority failed');
				case 6:
					return _('Message decrypted succesfully');
				case 7: 
					return _('Message decryption failed');
				case 8: 
					return _('Please unlock your certificate');
				case 9: 
					return _('Cannot determine revocation status of certificate');
				case 10:
					return _('Signature verified (OCSP disabled)');
				case 11:
					return _('Cannot determine revocation status of certificate');
			}
		},
		
		/**
		 * Function which returns the css class for the corresponding error code
		 *
		 * @param {Number} index
		 * @return {String} text 
		 */
		getStatusMessageClass : function(index)
		{
			switch(index) {
				case 0:
					return 'smime-info-good';
				case 1:
					return 'smime-info-partial';
				case 2:
					return 'smime-info-fatal';
			}
		},

		/**
		 * Function which returns the popup status for the corresponding error code
		 *
		 * @param {Number} index
		 * @return {String} text 
		 */
		getPopupStatus : function(index)
		{
			switch(index) {
				case 0:
					return '';
				case 1:
					return 'warning';
				case 2:
					return 'error';
			}
		}
	};
}();
Ext.namespace('Zarafa.plugins.smime.data');

/**
 * @class Zarafa.plugins.smime.data.JsonCertificateReader
 * @extends Zarafa.core.data.JsonReader
 */
Zarafa.plugins.smime.data.JsonCertificateReader = Ext.extend(Zarafa.core.data.JsonReader, {
        /**
         * @cfg {Zarafa.core.data.RecordCustomObjectType} customObjectType The custom object type
         * which represents the {@link Ext.data.Record records} which should be created using
         * {@link Zarafa.core.data.RecordFactory#createRecordObjectByCustomType}.
         */
        customObjectType : Zarafa.core.data.RecordCustomObjectType.ZARAFA_SMIME,

        /**
         * @constructor
         * @param {Object} meta Metadata configuration options.
         * @param {Object} recordType (optional) Optional Record type matches the type
         * which must be read from response. If no type is given, it will use the
         * record type for the {@link Zarafa.core.data.RecordCustomObjectType#ZARAFA_SMIME}.
         */
        constructor : function(meta, recordType)
        {
                meta = Ext.applyIf(meta || {}, {
                        dynamicRecord : false
                });

		recordType = Zarafa.core.data.RecordFactory.getRecordClassByCustomType(Zarafa.core.data.RecordCustomObjectType.ZARAFA_SMIME);

                Zarafa.plugins.smime.data.JsonCertificateReader.superclass.constructor.call(this, meta, recordType);
        }
});
Ext.namespace('Zarafa.plugins.smime.data');

/**
 * @class Zarafa.plugins.smime.data.SmimeAttachmentRecordFields
 * Array of {@link Ext.data.Field field} configurations for the
 * {@link Zarafa.plugins.smime.data.SmimeAttachmentRecord} object.
 */
Zarafa.plugins.smime.data.SmimeAttachmentRecordFields = [
	{name: 'cert', type: 'boolean', defaultValue: false}, 
	{name: 'cert_message', type: 'string'},
	{name: 'cert_warning', type: 'string'}
];

Zarafa.core.data.RecordCustomObjectType.addProperty('ZARAFA_SMIME_ATTACHMENT');
Zarafa.core.data.RecordFactory.setBaseClassToCustomType(Zarafa.core.data.RecordCustomObjectType.ZARAFA_SMIME_ATTACHMENT, Zarafa.plugins.smime.data.SmimeAttachmentRecord);
Zarafa.core.data.RecordFactory.addFieldToCustomType(Zarafa.core.data.RecordCustomObjectType.ZARAFA_SMIME_ATTACHMENT, Zarafa.plugins.smime.data.SmimeAttachmentRecordFields);
Zarafa.core.data.RecordFactory.addFieldToCustomType(Zarafa.core.data.RecordCustomObjectType.ZARAFA_SMIME_ATTACHMENT, Zarafa.core.data.IPMAttachmentRecordFields);
Ext.namespace('Zarafa.plugins.smime.data');

/**
 * @class Zarafa.plugins.smime.data.SmimeAttachmentStore
 * @extends Zarafa.core.data.IPMAttachmentStore
 * AttachmentStore specific for S/MIME Plugin which creates {@link Zarafa.plugins.smime.SmimeAttachmentRecord record}.
 * @private
 */
Zarafa.plugins.smime.data.SmimeAttachmentStore = Ext.extend(Zarafa.core.data.IPMAttachmentStore, {
	/**
	 * @cfg {Zarafa.core.data.RecordCustomObjectType} smime objecttype for the creation of our own attachmenttype in the attachmentstore
	 */
	attachmentRecordType : Zarafa.core.data.RecordCustomObjectType.ZARAFA_SMIME_ATTACHMENT,

	/**
	 * @constructor
	 * @param config Configuration object
	 */
	constructor : function(config)
	{
		config = config || {};

		Zarafa.plugins.smime.data.SmimeAttachmentStore.superclass.constructor.call(this, config);
	}
});

Ext.reg('smime.attachmentstore', Zarafa.plugins.smime.data.SmimeAttachmentStore);
Ext.namespace('Zarafa.plugins.smime.data');

/**
 * @class Zarafa.plugins.smime.data.SmimeCertificateRecordFields
 * Array of {@link Ext.data.Field field} configurations for the
 * {@link Zarafa.plugins.smime.data.SmimeCertificateRecord} object.
 */
Zarafa.plugins.smime.data.SmimeCertificateRecordFields = [
	{name: 'entryid', type: 'string'},
	{name: 'type', type: 'string'}, // Public or Private
	{name: 'issued_by', type: 'string'},
	{name: 'issued_to', type: 'string'},
	{name: 'serial', type: 'string'},
	{name: 'email', type: 'string', defaultValue: ''},
	{name: 'validto', type: 'date', dateFormat: 'timestamp', defaultValue: null},
	{name: 'validfrom', type: 'date', dateFormat: 'timestamp', defaultValue: null},
	{name: 'fingerprint_sha1', type: 'string'},
	{name: 'fingerprint_md5', type: 'string'}
];

/**
 * @class Zarafa.plugins.smime.data.SmimeCertificateRecord
 * @extends Zarafa.core.data.IPMRecord
 *
 * An extension to the {@link Zarafa.core.data.IPMRecord}.
 */
Zarafa.plugins.smime.data.SmimeCertificateRecord = Ext.extend(Zarafa.core.data.IPMRecord, {});

Zarafa.core.data.RecordCustomObjectType.addProperty('ZARAFA_SMIME');
Zarafa.core.data.RecordFactory.addFieldToCustomType(Zarafa.core.data.RecordCustomObjectType.ZARAFA_SMIME, Zarafa.plugins.smime.data.SmimeCertificateRecordFields);
Zarafa.core.data.RecordFactory.addFieldToCustomType(Zarafa.core.data.RecordCustomObjectType.MAPI_SMIME_ATTACH, Zarafa.core.data.IPMRecordFields);
Zarafa.core.data.RecordFactory.setSubStoreToCustomType(Zarafa.core.data.RecordCustomObjectType.ZARAFA_SMIME, 'attachments', Zarafa.plugins.smime.data.SmimeAttachmentStore);
Zarafa.core.data.RecordFactory.addListenerToCustomType(Zarafa.core.data.RecordCustomObjectType.ZARAFA_SMIME, 'createphantom', function(record)
{
	// Phantom records must always be marked as opened (they contain the full set of data)
	record.afterOpen();
});

Zarafa.core.data.RecordFactory.setBaseClassToCustomType(Zarafa.core.data.RecordCustomObjectType.ZARAFA_SMIME, Zarafa.plugins.smime.data.SmimeCertificateRecord);
Ext.namespace('Zarafa.plugins.smime.data');

/**
 * @class Zarafa.plugins.smime.data.SmimeCertificateStore
 * @extends Zarafa.core.data.ListModuleStore
 * @xtype smime.certificatestore
 * Store specific for S/MIME Plugin which creates {@link Zarafa.plugins.smime.SmimeCertificateRecord record}.
 */
Zarafa.plugins.smime.data.SmimeCertificateStore = Ext.extend(Zarafa.core.data.ListModuleStore, {
	/**
	 * @constructor
	 * @param config Configuration object
	 */
	constructor : function(config)
	{
		config = config || {};

		Ext.applyIf(config, {
			autoLoad : true,
			remoteSort: false,
			reader : new Zarafa.plugins.smime.data.JsonCertificateReader(),
			writer : new Zarafa.core.data.JsonWriter(),
			proxy  : new Zarafa.core.data.IPMProxy({
				listModuleName: 'pluginsmimemodule',
				itemModuleName: 'pluginsmimemodule'
			})
		});

		Zarafa.plugins.smime.data.SmimeCertificateStore.superclass.constructor.call(this, config);
	}
});

Ext.reg('smime.certificatestore', Zarafa.plugins.smime.data.SmimeCertificateStore);
Ext.namespace('Zarafa.plugins.smime.data');

/**
 * @class Zarafa.plugins.smime.data.SmimeResponseHandler
 * @extends Zarafa.core.data.AbstractResponseHandler
 *
 * Smime specific response handler.
 */
Zarafa.plugins.smime.data.SmimeResponseHandler = Ext.extend(Zarafa.core.data.AbstractResponseHandler, {

	/**
	 * @cfg {Function} successCallback The function which
	 * will be called after success request.
	 */
	successCallback : null,
	
	/**
	 * @param {Object} response Object contained the response data.
	 */
	doCertificate : function(response) {
		this.successCallback(response);
	},

	/**
	 * @param {Object} response Object contained the response data.
	 */
	doPassphrase : function(response) {
		this.successCallback(response);
	}
});

Ext.reg('smime.responsehandler', Zarafa.plugins.smime.data.SmimeResponseHandler);
Ext.namespace('Zarafa.plugins.smime.dialogs');

/**
 * @class Zarafa.plugins.smime.dialogs.PassphraseWindow
 * @extends Zarafa.core.ui.ContentPanel
 *
 * The content panel which asks the user for his passphrase and verifies if it's correct.
 * @xtype smime.passphrasewindow
 */
Zarafa.plugins.smime.dialogs.PassphraseWindow = Ext.extend(Zarafa.core.ui.ContentPanel, {

	/**
	 * cfg {Ext.Button} btn the smime security dropdown button
	 */
	btn : undefined,

	/**
	 * @constructor
	 * @param config Configuration structure
	 */
	constructor : function(config) {
		config = config || {};

		this.btn = config.record;

		Ext.applyIf(config, {
			layout : 'form',
			modal: true,
			width: 300,
			height: 300,
			title : _('S/MIME Passphrase'),
			xtype: 'smime.passphrasewindow',
			items: [{
				xtype: 'textfield',
				inputType: 'password',
				fieldLabel: _('Certificate passphrase'),
				ref: 'passphrase',
				scope: this,
				listeners: {
					scope: this,
					specialkey: this.onSpecialKey
				}
			},{
				xtype: 'button',
				text:  _('Submit'),
				handler: this.checkPassphrase,
				scope: this
			}]
		});
		Zarafa.plugins.smime.dialogs.PassphraseWindow.superclass.constructor.call(this, config);
	},
	
	/**
	 * Function which checks if the user inputs an enter in the password textfield
	 * And then checks if a valid passphrase has been entered. 
	 * @param {Ext.form.TextField} field
	 * @param {Ext.EventObject} eventobj
	 */
	onSpecialKey : function(field, eventobj)
	{
		if(eventobj.getKey() === eventobj.ENTER) {
			this.checkPassphrase();
		}
	},

	/**
	 * Function which calls a request to PHP which verifies if the supplied passphrase is correct.
	 * Calls onPassphraseCallback if there is a succesCallback.
	 * @param {Ext.button} button submit button of the form
	 */
	checkPassphrase : function(button) {
		var user = container.getUser();
		container.getRequest().singleRequest(
			'pluginsmimemodule',
			'passphrase',
			{ 
			  'user' : user.getSMTPAddress(),
			  'sessionid' : user.getSessionId(),
			  'passphrase' : this.passphrase.getValue()
			},
			new Zarafa.plugins.smime.data.SmimeResponseHandler({
				successCallback : this.onPassphraseCallback.createDelegate(this)
			})
		);
	},

	/**
	 * successCallback function for the request to verify if a private certificate passphrase is correct
	 * If the response status is true, the contentpanel will be closed and the record message_class will be set 
	 * and the record is saved.
	 * Otherwise the inputfield will be reset.
	 * @param {Object} response Json object containing the response from PHP
	 */
	onPassphraseCallback : function(response) {
		if(response.status) {
			if(this.btn instanceof Zarafa.core.data.IPMRecord) {
				this.btn.open({forceLoad: true});
			} else {
				var owner = this.btn.ownerCt;
				var record = owner.record;
				record.set('message_class', this.btn.message_class);
				record.save();
				this.btn.setIconClass('icon_smime_sign_selected')
			}
			this.close();
		} else {
			this.passphrase.reset();
			container.getNotifier().notify('error.connection', _('S/MIME Message'), _('Password incorrect'));
		}
	}
});

Ext.reg('smime.passphrasewindow', Zarafa.plugins.smime.dialogs.PassphraseWindow);
Ext.namespace('Zarafa.plugins.smime.settings');

/**
 * @class Zarafa.plugins.smime.settings.PublickeyGrid
 * @extends Ext.grid.GridPanel
 * @xtype smime.publickeygrid
 *
 * {@link Zarafa.plugins.smime.settings.PublickeyGrid PublickeyGrid} will be used to display
 * public certificates of the current user.
 */
Zarafa.plugins.smime.settings.PublickeyGrid = Ext.extend(Ext.grid.GridPanel, {
	/**
	 * @constructor
	 * @param {Object} config Configuration structure
	 */
	constructor : function(config)
	{
		config = config || {};

		if(!config.store) {
			config.store = new Zarafa.plugins.smime.data.SmimeCertificateStore();
		}
	
		Ext.applyIf(config, {
			xtype : 'smime.publickeygrid',
			store : config.store,
			viewConfig : {
				forceFit : true,
				emptyText : '<div class=\'emptytext\'>' + _('No certificates exists') + '</div>'
			},
			loadMask : this.initLoadMask(),
			columns : this.initColumnModel(),
			selModel : this.initSelectionModel(),
			listeners : {
				viewready : this.onViewReady,
				scope : this
			}
		});

		Zarafa.plugins.smime.settings.PublickeyGrid.superclass.constructor.call(this, config);
	},

	/**
	 * initialize events for the grid panel.
	 * @private
	 */
	initEvents : function()
	{
		Zarafa.plugins.smime.settings.PublickeyGrid.superclass.initEvents.call(this);

		// select first certificate when store has finished loading
		this.mon(this.store, 'load', this.onViewReady, this, {single : true});
	},

	/**
	 * Creates a column model object, used in {@link #colModel} config
	 * @return {Ext.grid.ColumnModel} column model object
	 * @private
	 */
	initColumnModel : function()
	{
		return [{
			dataIndex : 'email',
			header : _('Email'),
		    	sortable: true,
			renderer : Ext.util.Format.htmlEncode
		},{
			dataIndex : 'validfrom',
		    	sortable: true,
			header : _('Valid from'),
			renderer : Ext.util.Format.htmlEncode
		},{
			dataIndex : 'validto',
		    	sortable: true,
			header : _('Expires'),
			renderer : Ext.util.Format.htmlEncode
		},{
			dataIndex : 'type',
		    	sortable: true,
			header : _('Type'),
			renderer : Ext.util.Format.htmlEncode
		}];
	},

	/**
	 * Creates a selection model object, used in {@link #selModel} config
	 * @return {Ext.grid.RowSelectionModel} selection model object
	 * @private
	 */
	initSelectionModel : function()
	{
		return new Ext.grid.RowSelectionModel({
			singleSelect : true
		});
	},

	/**
	 * Initialize the {@link Ext.grid.GridPanel.loadMask} field
	 *
	 * @return {Object} The configuration object for {@link Ext.LoadMask}
	 * @private
	 */
	initLoadMask : function()
	{
		return {
			msg : _('Loading certificates') + '...'
		};
	},

	/**
	 * Event handler which is fired when the gridPanel is ready. This will automatically
	 * select the first row in the grid.
	 * @private
	 */
	onViewReady : function()
	{
		this.getSelectionModel().selectFirstRow();
	},



	/**
	 * Function will be called to remove a certificate.
	 */
	removeCertificate : function()
	{
		var selectionModel = this.getSelectionModel();
		var certificate = selectionModel.getSelected();

		if(!certificate) {
			Ext.Msg.alert(_('Alert'), _('Please select a certificate.'));
			return;
		} else if(certificate.get('type') === 'private') {
			Ext.MessageBox.show({
				title: _('S/MIME'),
				msg :_('Do you really want to remove your private certificate? If you remove your certificate you will not be able to sign or decrypt S/MIME emails.'),
				icon: Ext.MessageBox.WARNING,
				fn: this.onRemoveCertificate,
				scope: this,
				buttons: Ext.MessageBox.YESNO
			});
		} else {
			this.onRemoveCertificateHelper();
		}
	},	
	
	/**
	 * Handler for removing private certificate dialog
	 * @param {String} btn 
	 */
	onRemoveCertificate : function(btn)
	{
		if(btn === 'yes') {
			this.onRemoveCertificateHelper();
		}
	},

	/**
	 * Helper function to avoid code duplication in onRemoveCertificate and removeCertificate
	 */
	onRemoveCertificateHelper : function()
	{
		var selectionModel = this.getSelectionModel();
		var certificate = selectionModel.getSelected();
		// before removing record we should select next available record,
		// because deleting record will remove selection
		if (selectionModel.hasNext()) {
			selectionModel.selectNext();
		} else if (selectionModel.hasPrevious()) {
			selectionModel.selectPrevious();
		}

		this.store.remove(certificate);
	},

	/**
	 * Function will be called to show details of a certificate
	 */
	showDetails : function()
	{
		var selectionModel = this.getSelectionModel();
		var certificate = selectionModel.getSelected();
		if(Ext.isDefined(certificate)) {
			// TODO: use ExtJS form?
			
			var text = "Email: " + certificate.get('email') + "</br>" 
				+ "Serial: " + certificate.get('serial') + "</br>" 
				+ "Issued by: " + certificate.get('issued_by') + "</br>"
				+ "Issued to: " + certificate.get('issued_to') + "</br>"
				+ "SHA1 Fingerprint: " + certificate.get('fingerprint_sha1') + "</br>"
				+ "MD5 Fingerprint: " + certificate.get('fingerprint_md5') + "</br>";
			Ext.Msg.alert(_('Certificate details'), text);
		}
	}

});

Ext.reg('smime.publickeygrid', Zarafa.plugins.smime.settings.PublickeyGrid);
Ext.namespace('Zarafa.plugins.smime.settings');

/**
 * @class Zarafa.plugins.smime.settings.SettingsPublickeyPanel
 * @extends Ext.Panel
 * @xtype smime.publiccertificatespanel
 * Will generate UI for the {@link Zarafa.plugins.smime.settings.SettingsPublickeyWidget SettingsPublickeyWidget}.
 */
Zarafa.plugins.smime.settings.SettingsPublickeyPanel = Ext.extend(Ext.Panel, {
	/**
	 * @cfg {Zarafa.plugins.smime.data.SmimeCertificateStore} store Certificate store that will be used to load public certificates
	 */
	store : undefined,

	/**
	 * The LoadMask object which will be shown when the {@link #record} is being opened, and
	 * the dialog is waiting for the server to respond with the desired data.
	 * @property
	 * @type Zarafa.common.ui.LoadMask
	 */
	loadMask : undefined,

	/**
	 * @constructor
	 * @param config Configuration structure
	 */
	constructor : function(config)
	{
		config = config || {};

		if(!config.store) {
			config.store = new Zarafa.plugins.smime.data.SmimeCertificateStore();
		}

		Ext.applyIf(config, {
			xtype : 'smime.publiccertificatespanel',
			border : false,
			layout : {
				type : 'vbox',
				align : 'stretch',
				pack  : 'start'
			},
			items : this.createPanelItems(config.store)
		});

		Zarafa.plugins.smime.settings.SettingsPublickeyPanel.superclass.constructor.call(this, config);
	},

	/**
	 * Function will create panel items for {@link Zarafa.plugins.smime.data.SmimeCertificateStore SmimeCertificateStore}
	 * @return {Array} array of items that should be added to panel.
	 * @param {Zarafa.plugins.smime.data.SmimeCertificateStore} store store that will be used to load public certificates.
	 * @private
	 */
	createPanelItems : function(store)
	{
		var certStore = store;
		return [{
			xtype : 'displayfield',
			value : _('Below you can view and delete public & private certificate(s) stored on the server.'),
			fieldClass : 'x-form-display-field zarafa-settings-widget-extrainfo'
		}, {
			xtype : 'container',
			flex : 1,
			layout : {
				type : 'hbox',
				align : 'stretch',
				pack  : 'start'
			},
			items : [{
				xtype : 'smime.publickeygrid',
				ref : '../publickeyGrid',
				store : store,
				flex : 1
			}, {
				xtype : 'container',
				width : 160,
				defaults : {
					width : 140
				},
				layout : {
					type : 'vbox',
					align : 'center',
					pack  : 'start'
				},
				items : [{
					xtype : 'displayfield',
					value : _('Filter on Email')
				}, {
					xtype : 'textfield',
					listeners : {
						specialkey: function(f, e) {
							if(e.getKey() == e.ENTER){
								certStore.filter('email', f.getValue());
							}
						}
					}
				}, {
					xtype : 'spacer',
					height : 20
				}, {
					xtype : 'button',
					text : _('Remove'),
					disabled : true,
					ref : '../../removeButton',
					handler : this.onCertificateRemove,
					scope : this
				}, {
					xtype : 'button',
					text : _('Details'),
					disabled : true,
					ref : '../../detailButton',
					handler : this.onDetailButton,
					scope : this

				}]
			}]
		}];
	},

	/**
	 * initialize events for the panel.
	 * @private
	 */
	initEvents : function()
	{
		Zarafa.plugins.smime.settings.SettingsPublickeyPanel.superclass.initEvents.call(this);

		// register event to enable/disable buttons
		this.mon(this.publickeyGrid.getSelectionModel(), 'selectionchange', this.onGridSelectionChange, this);
	},

	/**
	 * If {@link #showLoadMask} is enabled, this function will display the {@link #loadMask}.
	 * @protected
	 */
	showLoadMask : function()
	{
		if (!this.loadMask) {
			this.loadMask = new Zarafa.common.ui.LoadMask(this.el);
		}

		this.loadMask.show();
	},

	/**
	 * If {@link #showLoadMask} is enabled, and {@link #showLoadMask} has been
	 * called to display the {@link #loadMask} this function will disable the loadMask.
	 * @protected
	 */
	hideLoadMask : function()
	{
		if (this.loadMask) {
			this.loadMask.hide();
		}
	},

	/**
	 * Event handler will be called when selection in {@link Zarafa.plugins.smime.settings.PublickeyGrid PublickeyGrid}
	 * has been changed
	 * @param {Ext.grid.RowSelectionModel} selectionModel selection model that fired the event
	 */
	onGridSelectionChange : function(selectionModel)
	{
		var noSelection = (selectionModel.hasSelection() === false);

		this.removeButton.setDisabled(noSelection);
		this.detailButton.setDisabled(noSelection);
	},

	/**
	 * Handler function will be called when user clicks on 'Remove' button,
	 * this will remove currently selected certificate from certificates list.
	 * @private
	 */
	onCertificateRemove : function()
	{
		this.publickeyGrid.removeCertificate();
	},

	/**
	 * Handler function will be called when user clicks on 'Remove' button,
	 * this will remove currently selected certificate from certificates list.
	 * @private
	 */
	onDetailButton : function()
	{
		this.publickeyGrid.showDetails();
	},

	/**
	 * Function will be used to reload data in the {@link Zarafa.plugins.smime.data.SmimeCertificateStore SmimeCertificateStore}
	 */
	discardChanges : function()
	{
		this.store.load();
	},

	/**
	 * Function will be used to save changes in the {@link Zarafa.plugins.smime.data.SmimeCertificateStore SmimeCertificateStore}
	 */
	saveChanges : function()
	{
		this.store.save();
	}
});

Ext.reg('smime.publiccertificatespanel', Zarafa.plugins.smime.settings.SettingsPublickeyPanel);
Ext.namespace('Zarafa.plugins.smime.settings');
/**
 * @class Zarafa.smime.settings.SettingsPublickeyWidget
 * @extends Zarafa.settings.ui.SettingsWidget
 * @xtype smime.settingspublickeysmimewidget
 *
 * The {@link Zarafa.settings.SettingsPublickeyWidget widget} for managing public certificates
 */
Zarafa.plugins.smime.settings.SettingsPublickeyWidget = Ext.extend(Zarafa.settings.ui.SettingsWidget, {
	/**
	 * @constructor
	 * @param {Object} config Configuration object
	 */
	constructor : function(config) {
		config = config || {};

		Ext.applyIf(config, {
			height : 400,
			title : _('Public & Private certificates'),
			xtype : 'smime.settingspublickeysmimewidget',
			layout : {
				// override from SettingsWidget
				type : 'fit'
			},
			items : [{
				xtype : 'smime.publiccertificatespanel',
				ref : 'certificatesPanel',
				store: config.store
			}]
		});
		
		Zarafa.plugins.smime.settings.SettingsPublickeyWidget.superclass.constructor.call(this, config);
	},

	/**
	 * @return {Zarafa.public.smime.data.SmimeCertificateStore} The store which is holds all the public certificates
	 */
	getCertificateStore : function()
	{
		return this.certificatesPanel.store;
	},

	/**
	 * initialize events for the {@link Zarafa.plugins.smime.settings.SettingsPublickeyWidget SettingsPublickeyWidget}.
	 * @private
	 */
	initEvents : function()
	{
		Zarafa.plugins.smime.settings.SettingsPublickeyWidget.superclass.initEvents.call(this);

		// listen to savesettings and discardsettings to save/discard public certificates.
		var contextModel = this.settingsContext.getModel();

		this.mon(contextModel, 'savesettings', this.onSaveSettings, this);
		this.mon(contextModel, 'discardsettings', this.onDiscardSettings, this);

		this.mon(this.getCertificateStore(), {
			'remove' : this.doStoreRemove,
			'update' : this.doStoreUpdate,
			scope : this
		});
	},

	/**
	 * Event handler for the {@link Ext.data.Store#remove} event which is fired
	 * by the {@link Ext.data.Store} inside the {@link #certificatePanel}.
	 * This will mark the {@link Zarafa.settings.SettingsContextModel} as
	 * {@link Zarafa.settings.SettingsContextModel#setDirty dirty}.
	 * @param {Ext.data.Store} store The store which fired the event
	 * @param {Ext.data.Record} record The record which was updated
	 * @private
	 */
	doStoreRemove : function(store, record)
	{
		if(!record.phantom) {
			this.settingsContext.getModel().setDirty();
		}
	},

	/**
	 * Event handler for the {@link Ext.data.Store#update} event which is fired
	 * by the {@link Ext.data.Store} inside the {@link #certificatePanel}.
	 * This will mark the {@link Zarafa.settings.SettingsContextModel} as
	 * {@link Zarafa.settings.SettingsContextModel#setDirty dirty}.
	 * @param {Ext.data.Store} store The store which fired the event
	 * @param {Ext.data.Record} record The record which was updated
	 * @param {String} operation The update operation being performed.
	 * @private
	 */
	doStoreUpdate : function(store, record, operation)
	{
		if (operation !== Ext.data.Record.COMMIT) {
			this.settingsContext.getModel().setDirty();
		}
	},

	/**
	 * Event handler will be called when {@link Zarafa.settings.SettingsContextModel#savesettings} event is fired.
	 * This will relay this event to {@link Zarafa.plugins.smime.settings.SettingsPublickeyPanel PublickeyPanel} so it can
	 * save certificates.
	 * @private
	 */
	onSaveSettings : function()
	{
		this.certificatesPanel.saveChanges();
	},

	/**
	 * Event handler will be called when {@link Zarafa.settings.SettingsContextModel#discardsettings} event is fired.
	 * This will relay this event to {@link Zarafa.plugins.smime.settings.SettingsPublicKeyPanel PublickeyPanel} so it can
	 * discard current changes and reload public certificates from the server.
	 * @private
	 */
	onDiscardSettings : function()
	{
		this.certificatesPanel.discardChanges();
	}
});

Ext.reg('smime.settingspublickeywidget', Zarafa.plugins.smime.settings.SettingsPublickeyWidget);
Ext.namespace('Zarafa.plugins.smime.settings');

/**
 * @class Zarafa.plugins.smime.settings.SettingsSmimeCategory
 * @extends Zarafa.settings.ui.SettingsCategory
 * @xtype smime.settingssmimecategory
 *
 * The smime category for users which will
 * allow the user to upload public/private certificates
 */
Zarafa.plugins.smime.settings.SettingsSmimeCategory = Ext.extend(Zarafa.settings.ui.SettingsCategory, {
        /**
         * @insert context.settings.category.smime
         * Insertion point to register new {@link Zarafa.settings.settingssmimewidget widgets}
         * for the {@link Zarafa.smime.settings.settingssmimecategory}.
         * @param {Zarafa.smime.settings.settingssmimecategory} category The smime
         * category to which the widgets will be added.
         */

        /**
         * @constructor
         * @param {Object} config Configuration object
         */
        constructor : function(config) {
                config = config || {};
		this.store = new Zarafa.plugins.smime.data.SmimeCertificateStore();

                Ext.applyIf(config, {
			title : _('S/MIME'),
                        categoryIndex : 1,
                        iconCls : 'icon_smime_settings',
                        items : [{
				xtype : 'smime.settingssmimewidget',
				store: this.store
                        },{
				xtype : 'smime.uploadcertificatewidget',
				store: this.store
                        },{
				xtype : 'smime.settingspublickeywidget',
				settingsContext : config.settingsContext,
				store: this.store
			}]
                });

                Zarafa.plugins.smime.settings.SettingsSmimeCategory.superclass.constructor.call(this, config);
        },

	/**
	 * Called by superclass when the Category has been deselected and is hidden from the user,
	 * this will unregister the {@link #onBeforeSaveRules} event handler.
	 * @private
	 */
	onHide : function()
	{
                Zarafa.plugins.smime.settings.SettingsSmimeCategory.superclass.onHide.apply(this, arguments);

		// Unregister the 'beforesave' event. This could be lingering when
		// 'savesettings' was fired but it was cancelled by one of the
		// event handlers.
		this.mun(this.store, 'beforesave', this.onBeforeSaveCertificate, this);
	},

	/**
	 * Event handler for the
	 * {@link Zarafa.settings.SettingsContextModel ContextModel}#{@link Zarafa.settings.SettingsContextModel#beforesavesettings beforesavesettings}
	 * event. It will reset the {@link #savingElCounter} and his will register the event handler for
	 * {@link Zarafa.settings.SettingsModel#beforesave beforesave} event.
	 * @private
	 */
	onBeforeSaveSettingsModel : function()
	{
		Zarafa.plugins.smime.settings.SettingsSmimeCategory.superclass.onBeforeSaveSettingsModel.apply(this, arguments);

		this.mon(this.store, 'beforesave', this.onBeforeSaveCertificate, this, { single : true });
	},
	
	/**
	 * Event handler which is fired when the {@link Zarafa.plugins.smime.data.SmimeCertificateStore SmimeCertificateStore}
	 * fires the 'beforesave' event. This will {@link #displaySavingMask show a notification} and register the
	 * event handlers for the completion of the save.
	 * @private
	 */
	onBeforeSaveCertificate : function()
	{
		this.displaySavingMask();

		this.mon(this.store, 'save', this.onCertificateSave, this);
	},

	/**
	 * Event handler which is fired when the {@link Zarafa.plugins.smime.data.SmimeCertificateStore SmimeCertificateStore}
	 * fires the 'save' event indicating the successfull save of the delegates. This will
	 * {@link #hideSavingMask hide the notification}.
	 * @private
	 */
	onCertificateSave : function()
	{
		this.hideSavingMask(true);

		this.mun(this.store, 'save', this.onCertificateSave, this);
	}
});

Ext.reg('smime.settingssmimecategory', Zarafa.plugins.smime.settings.SettingsSmimeCategory); 
Ext.namespace('Zarafa.plugins.smime.settings');
/**
 * @class Zarafa.plugins.smime.settings.SettingsSmimeWidget
 * @extends Zarafa.settings.ui.SettingsWidget
 * @xtype smime.settingssmimewidget
 *
 * The {@link Zarafa.plugins.settings.SettingsSmimeWidget widget} for importing S/MIME certificates (public/private)
 */
Zarafa.plugins.smime.settings.SettingsSmimeWidget = Ext.extend(Zarafa.settings.ui.SettingsWidget, {

	/**
	 * @cfg {Zarafa.core.data.IPMRecord} record.
	 */
	record : undefined,

	/**
	 * @constructor
	 * @param {Object} config Configuration object
	 */
	constructor : function(config) {
		config = config || {};

		if(!config.store) {
			config.store = new Zarafa.plugins.smime.data.SmimeCertificateStore();
		}

		Ext.applyIf(config, {
			title	: _('Personal certificate'),
			layout : 'form',
			xtype : 'smime.settingssmimewidget',
			items :[{
				xtype: 'displayfield',
				ref: 'certificateField',
				hideLabel : true,
				defaultValue : _('You don\'t have a valid certificate corresponding to your account')
			}]
		});

		Zarafa.plugins.smime.settings.SettingsSmimeWidget.superclass.constructor.call(this, config);
	},


	/**
	 * initialize events for the grid panel.
	 * @private
	 */
	initEvents : function()
	{
		Zarafa.plugins.smime.settings.SettingsSmimeWidget.superclass.initEvents.call(this);
		// TODO: shouldn't this be update?
		this.mon(this.store, 'load', this.onStoreReady, this);
		this.mon(this.store, 'remove', this.onStoreReady, this);
		this.onStoreReady();
	},

	/**
	 * Event handler which is fired when the store is loaded or an item is removed from the store
	 * @private
	 */
	onStoreReady : function()
	{
		var index = this.store.findExact('type', 'private')
		if(index === -1) {
			this.certificateField.setRawValue(this.certificateField.defaultValue);
		} else {
			this.record = this.store.getAt(index);
			// TODO: add validity message
			this.certificateField.setRawValue(_('You have a valid certificate corresponding to your account'));
		}
	}
});

Ext.reg('smime.settingssmimewidget', Zarafa.plugins.smime.settings.SettingsSmimeWidget);
Ext.namespace('Zarafa.plugins.smime.settings');
/*
 * #dependsFile plugins/smime/js/data/SmimeAttachmentStore.js
 */

/**
 * @class Zarafa.plugins.smime.settings.UploadCertificateWidget
 * @extends Zarafa.settings.ui.SettingsWidget
 * @xtype smime.uploadcertificatewidget
 *
 * The {@link Zarafa.plugins.settings.UploadCertificateWidget widget} for importing S/MIME certificates (public/private)
 */
Zarafa.plugins.smime.settings.UploadCertificateWidget = Ext.extend(Zarafa.settings.ui.SettingsWidget, {
	/**
	 * @cfg {Object} form used to temprorarly store the value of form in 
	 * {@link #selectCertificateCallback} for {@link #uploadCertificate}.
	 */
	form: undefined,

	/**
	 * @cfg {Object} files used to temprorarly store the value of form in 
	 * {@link #selectCertificateCallback for {@link #uploadCertificate}.
	 */
	files: undefined,

	/**
	 * @cfg {Zarafa.core.data.IPMRecord} record.
	 */
	record : undefined,

	/**
	 * @constructor
	 * @param {Object} config Configuration object
	 */
	constructor : function(config) {
		config = config || {};

		if(!config.store) {
			config.store = new Zarafa.plugins.smime.data.SmimeCertificateStore();
		}

		Ext.applyIf(config, {
			title	: _('Upload your certificate'),
			layout : 'form',
			xtype : 'smime.uploadcertificatewidget',
			items :[{
				xtype: 'displayfield',
				hideLabel : true,
				value : _('Below you can upload your private certificate in PKCS#12 format. The WebApp only accepts certificates which are valid and only stores one private certificate on the server.')
			},{
				xtype : 'button',
				text : _('Select'),
				defaultValue : _('Select'),
				iconCls : 'icon_smime_settings',
				fieldLabel : _('Private certificate'),
				ref : 'certificate',
				width: 200,
				handler : this.selectCertificate,
				scope : this
			},{
				xtype : 'textfield',
				inputType: 'password',
				fieldLabel : _('Certificate passphrase'),
				width: 200,
				ref : 'passphrase',
				scope : this
			},{
				xtype : 'button',
				text : _('Upload'),
				width : 100,
				handler : this.uploadCertificate,
				scope : this
			}]
		});

		Zarafa.plugins.smime.settings.UploadCertificateWidget.superclass.constructor.call(this, config);
	},

	/**
	 * Handler for {Ext.button} opens a attachment dialog where the user can select a certificate.
	 * Registers a callback function which receives the files and form.
	 */
	selectCertificate : function() 
	{
		this.record = Zarafa.core.data.RecordFactory.createRecordObjectByCustomType(Zarafa.core.data.RecordCustomObjectType.ZARAFA_SMIME, {});

		var attachComponent = new Zarafa.common.attachment.ui.UploadAttachmentComponent({
			callback : this.selectCertificateCallback,
			scope : this
		});

		attachComponent.openAttachmentDialog();
	},

	/**
	 * Handler for {Ext.button} 'upload', uploads certificate if a passphrase is filled in
	 * and certificate is selected.
	 */
	uploadCertificate : function()
	{
		if(Ext.isEmpty(this.passphrase.getValue())) {
			Ext.MessageBox.alert(_('Alert'), _('Passphrase is empty'));
		} else if(Ext.isEmpty(this.files)) {
			Ext.MessageBox.alert(_('Alert'), _('No certificate selected'));
		} else {
			var attachmentStore = this.record.getAttachmentStore();
			this.mon(attachmentStore, 'update', this.onUpdate, this);

			var param = {sourcetype : 'certificate', passphrase: this.passphrase.getValue() };
			attachmentStore.uploadFiles(this.files, this.form, true, param);
		}
	},

	/**
	 * Callback function for {@link Zarafa.plugins.smime.settingssmimewidget.selectCertificate}.
	 * Function is used to set the files and form for {@link Zarafa.plugins.smime.settingsmimewidget.uploadCertificate}
	 * It also sets the certificate upload button's text to the selected filename.
	 * 
	 * @param {Object/Array} files The files is contains file information.
	 * @param {Object} form the form is contains {@link Ext.form.BasicForm bacisform} info.
	 */
	selectCertificateCallback : function(files, form)
	{
		this.certificate.setText(files[0].name);
		this.files = files;
		this.form = form;
	},

	/**
	 * Function for update event of {@link Zarafa.core.data.IPMAttachmentStore}.
	 * Displays the information which is stored in the {Ext.data.Record} fields called
	 * cert_warning and cert_message.
	 *
	 * @param {Ext.data.Store} store The {Ext.data.Store} where the record is stored.
	 * @param {Ext.data.record} record The {Ext.data.Record} from which the data was extracted.
	 */
	onUpdate : function(store, record)
	{
		if(record.get('cert')) {
			container.getNotifier().notify('info.saved', _('S/MIME Message'), record.get('cert_warning'));
			this.store.load();
			this.passphrase.reset();
			this.certificate.setText(this.certificate.defaultValue);
		} else {
			container.getNotifier().notify('error.connection', _('S/MIME Message'), record.get('cert_warning'));
		} 

	},

	/**
	 * Called by the {@link Zarafa.settings.ui.SettingsCategory Category} when
	 * it has been called with {@link zarafa.settings.ui.SettingsCategory#update}.
	 * This is being used to reset the textfield and button.
	 *
	 * @param {Zarafa.settings.SettingsModel} settingsModel The settings to load
	 */
	update : function(settingsModel)
	{
		this.passphrase.reset();
		this.certificate.setText(this.certificate.defaultValue);
	}	
});

Ext.reg('smime.uploadcertificatewidget', Zarafa.plugins.smime.settings.UploadCertificateWidget);
